@@ -1,9 +1,10 @@
 """Java code generator for a given schema salad definition."""
 import os
+import re
 import shutil
 import string
 from io import StringIO
-from io import open as io_open
+from pathlib import Path
 from typing import (
     Any,
     Dict,
@@ -14,11 +15,10 @@ from typing import (
     Set,
     Union,
 )
-from urllib.parse import urlsplit
 
 import pkg_resources
 
-from . import schema
+from . import _logger, schema
 from .codegen_base import CodeGenBase, TypeDef
 from .exceptions import SchemaException
 from .schema import shortname
@@ -29,11 +29,13 @@ from .schema import shortname
 # may even confuse things a bit so turning these off be default.
 USE_ONE_OR_LIST_OF_TYPES = False
 
+BASIC_JAVA_IDENTIFIER_RE = re.compile(r"[^0-9a-zA-Z]+")
 
-def _ensure_directory_and_write(path: str, contents: str) -> None:
-    dirname = os.path.dirname(path)
-    _safe_makedirs(dirname)
-    with io_open(path, mode="w", encoding="utf-8") as f:
+
+def _ensure_directory_and_write(path: Path, contents: str) -> None:
+    _safe_makedirs(path.parent)
+    with open(path, mode="w", encoding="utf-8") as f:
+        _logger.info("Writing file: %s", path)
         f.write(contents)
 
 
@@ -48,61 +50,78 @@ def doc_to_doc_string(doc: Optional[str], indent_level: int = 0) -> str:
     return doc_str
 
 
-def _safe_makedirs(path):
-    # type: (str) -> None
-    if not os.path.exists(path):
+def _safe_makedirs(path: Path) -> None:
+    if not path.exists():
         os.makedirs(path)
+        _logger.info("Created directory: %s", path)
 
 
+_string_type_def = TypeDef(
+    instance_type="String",
+    init="new PrimitiveLoader<String>(String.class)",
+    name="StringInstance",
+    loader_type="Loader<String>",
+)
+
+_int_type_def = TypeDef(
+    instance_type="Integer",
+    init="new PrimitiveLoader<Integer>(Integer.class)",
+    name="IntegerInstance",
+    loader_type="Loader<Integer>",
+)
+
+_long_type_def = TypeDef(
+    instance_type="Long",
+    name="LongInstance",
+    loader_type="Loader<Long>",
+    init="new PrimitiveLoader<Long>(Long.class)",
+)
+
+_float_type_def = TypeDef(
+    instance_type="Double",
+    name="DoubleInstance",
+    loader_type="Loader<Double>",
+    init="new PrimitiveLoader<Double>(Double.class)",
+)
+
+_bool_type_def = TypeDef(
+    instance_type="Boolean",
+    name="BooleanInstance",
+    loader_type="Loader<Boolean>",
+    init="new PrimitiveLoader<Boolean>(Boolean.class)",
+)
+
+_null_type_def = TypeDef(
+    instance_type="Object",
+    name="NullInstance",
+    loader_type="Loader<Object>",
+    init="new NullLoader()",
+)
+
+_any_type_def = TypeDef(
+    instance_type="Object",
+    name="AnyInstance",
+    init="new AnyLoader()",
+    loader_type="Loader<Object>",
+)
+
 prims = {
-    "http://www.w3.org/2001/XMLSchema#string": TypeDef(
-        instance_type="String",
-        init="new PrimitiveLoader<String>(String.class)",
-        name="StringInstance",
-        loader_type="Loader<String>",
-    ),
-    "http://www.w3.org/2001/XMLSchema#int": TypeDef(
-        instance_type="Integer",
-        init="new PrimitiveLoader<Integer>(Integer.class)",
-        name="IntegerInstance",
-        loader_type="Loader<Integer>",
-    ),
-    "http://www.w3.org/2001/XMLSchema#long": TypeDef(
-        instance_type="Long",
-        name="LongInstance",
-        loader_type="Loader<Long>",
-        init="new PrimitiveLoader<Long>(Long.class)",
-    ),
-    "http://www.w3.org/2001/XMLSchema#float": TypeDef(
-        instance_type="Double",
-        name="DoubleInstance",
-        loader_type="Loader<Double>",
-        init="new PrimitiveLoader<Double>(Double.class)",
-    ),
-    "http://www.w3.org/2001/XMLSchema#double": TypeDef(
-        instance_type="Double",
-        name="DoubleInstance",
-        loader_type="Loader<Double>",
-        init="new PrimitiveLoader<Double>(Double.class)",
-    ),
-    "http://www.w3.org/2001/XMLSchema#boolean": TypeDef(
-        instance_type="Boolean",
-        name="BooleanInstance",
-        loader_type="Loader<Boolean>",
-        init="new PrimitiveLoader<Boolean>(Boolean.class)",
-    ),
-    "https://w3id.org/cwl/salad#null": TypeDef(
-        instance_type="Object",
-        name="NullInstance",
-        loader_type="Loader<Object>",
-        init="new NullLoader()",
-    ),
-    "https://w3id.org/cwl/salad#Any": TypeDef(
-        instance_type="Object",
-        name="AnyInstance",
-        init="new AnyLoader()",
-        loader_type="Loader<Object>",
-    ),
+    "http://www.w3.org/2001/XMLSchema#string": _string_type_def,
+    "http://www.w3.org/2001/XMLSchema#int": _int_type_def,
+    "http://www.w3.org/2001/XMLSchema#long": _long_type_def,
+    "http://www.w3.org/2001/XMLSchema#float": _float_type_def,
+    "http://www.w3.org/2001/XMLSchema#double": _float_type_def,
+    "http://www.w3.org/2001/XMLSchema#boolean": _bool_type_def,
+    "https://w3id.org/cwl/salad#null": _null_type_def,
+    "https://w3id.org/cwl/salad#Any": _any_type_def,
+    "string": _string_type_def,
+    "int": _int_type_def,
+    "long": _long_type_def,
+    "float": _float_type_def,
+    "double": _float_type_def,
+    "boolean": _bool_type_def,
+    "null": _null_type_def,
+    "Any": _any_type_def,
 }
 
 
@@ -112,34 +131,22 @@ class JavaCodeGen(CodeGenBase):
         base: str,
         target: Optional[str],
         examples: Optional[str],
-        package: Optional[str],
+        package: str,
         copyright: Optional[str],
     ) -> None:
         super().__init__()
         self.base_uri = base
-        sp = urlsplit(base)
         self.examples = examples
-        self.package = (
-            package
-            if package
-            else ".".join(
-                list(reversed(sp.netloc.split("."))) + sp.path.strip("/").split("/")
-            )
-        )
+        self.package = package
         self.artifact = self.package.split(".")[-1]
         self.copyright = copyright
-        target = target or "."
-        self.target_dir = target
+        self.target_dir = Path(target or ".").resolve()
         rel_package_dir = self.package.replace(".", "/")
-        self.rel_package_dir = rel_package_dir
-        self.main_src_dir = os.path.join(
-            self.target_dir, "src", "main", "java", rel_package_dir
-        )
-        self.test_src_dir = os.path.join(
-            self.target_dir, "src", "test", "java", rel_package_dir
-        )
-        self.test_resources_dir = os.path.join(
-            self.target_dir, "src", "test", "resources", rel_package_dir
+        self.rel_package_dir = Path(rel_package_dir)
+        self.main_src_dir = self.target_dir / "src" / "main" / "java" / rel_package_dir
+        self.test_src_dir = self.target_dir / "src" / "test" / "java" / rel_package_dir
+        self.test_resources_dir = (
+            self.target_dir / "src" / "test" / "resources" / rel_package_dir
         )
 
     def prologue(self) -> None:
@@ -181,7 +188,7 @@ class JavaCodeGen(CodeGenBase):
         self.current_class = cls
         self.current_class_is_abstract = abstract
         self.current_loader = StringIO()
-        self.current_fieldtypes = {}  # type: Dict[str, TypeDef]
+        self.current_fieldtypes: Dict[str, TypeDef] = {}
         self.current_fields = StringIO()
         interface_doc_str = f"* Auto-generated interface for <I>{classname}</I><BR>"
         if not abstract:
@@ -192,16 +199,17 @@ class JavaCodeGen(CodeGenBase):
             f"* Auto-generated class implementation for <I>{classname}</I><BR>"
         )
         class_doc_str += doc_to_doc_string(doc)
-        with open(os.path.join(self.main_src_dir, f"{cls}.java"), "w") as f:
-
+        target = self.main_src_dir / f"{cls}.java"
+        with open(target, "w") as f:
+            _logger.info("Writing file: %s", target)
             if extends:
                 ext = (
                     "extends "
                     + ", ".join(self.interface_name(e) for e in extends)
-                    + ", Savable"
+                    + ", Saveable"
                 )
             else:
-                ext = "extends Savable"
+                ext = "extends Saveable"
             f.write(
                 """// Copyright Common Workflow Language project contributors
 """
@@ -229,7 +237,7 @@ class JavaCodeGen(CodeGenBase):
 
 package {package};
 
-import {package}.utils.Savable;
+import {package}.utils.Saveable;
 
 /**
 {interface_doc_str}
@@ -245,7 +253,9 @@ public interface {cls} {ext} {{""".format(
         if self.current_class_is_abstract:
             return
 
-        with open(os.path.join(self.main_src_dir, f"{cls}Impl.java"), "w") as f:
+        target = self.main_src_dir / f"{cls}Impl.java"
+        with open(target, "w") as f:
+            _logger.info("Writing file: %s", target)
             f.write(
                 """// Copyright Common Workflow Language project contributors
 """
@@ -276,13 +286,13 @@ package {package};
 import {package}.utils.LoaderInstances;
 import {package}.utils.LoadingOptions;
 import {package}.utils.LoadingOptionsBuilder;
-import {package}.utils.SavableImpl;
+import {package}.utils.SaveableImpl;
 import {package}.utils.ValidationException;
 
 /**
 {class_doc_str}
  */
-public class {cls}Impl extends SavableImpl implements {cls} {{
+public class {cls}Impl extends SaveableImpl implements {cls} {{
   private LoadingOptions loadingOptions_ = new LoadingOptionsBuilder().build();
   private java.util.Map<String, Object> extensionFields_ =
       new java.util.HashMap<String, Object>();
@@ -329,11 +339,9 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
             )
         )
 
-    def end_class(self, classname, field_names):
-        # type: (str, List[str]) -> None
-        with open(
-            os.path.join(self.main_src_dir, f"{self.current_class}.java"), "a"
-        ) as f:
+    def end_class(self, classname: str, field_names: List[str]) -> None:
+        """Finish this class."""
+        with open(self.main_src_dir / f"{self.current_class}.java", "a") as f:
             f.write(
                 """
 }
@@ -360,9 +368,9 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
             )
 
         self.current_loader.write("""  }""")
-
+        target = self.main_src_dir / f"{self.current_class}Impl.java"
         with open(
-            os.path.join(self.main_src_dir, f"{self.current_class}Impl.java"),
+            target,
             "a",
         ) as f:
             f.write(self.current_fields.getvalue())
@@ -441,11 +449,16 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
                 "https://w3id.org/cwl/salad#array",
             ):
                 i = self.type_loader(type_declaration["items"])
+                instance_type = (
+                    "java.util.List<String>"
+                    if i.instance_type == "String"
+                    else "java.util.List<Object>"
+                )
                 return self.declare_type(
                     TypeDef(
                         # special doesn't work out with subclassing, gotta be more clever
                         # instance_type="List<{}>".format(i.instance_type),
-                        instance_type="java.util.List<Object>",
+                        instance_type=instance_type,
                         name=f"array_of_{i.name}",
                         init=f"new ArrayLoader({i.name})",
                         loader_type="Loader<java.util.List<{}>>".format(
@@ -496,8 +509,9 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
         symbols_decl = 'new String[] {{"{}"}}'.format(
             '", "'.join(sym for sym in symbols)
         )
-        enum_path = os.path.join(self.main_src_dir, f"{clazz}.java")
+        enum_path = self.main_src_dir / f"{clazz}.java"
         with open(enum_path, "w") as f:
+            _logger.info("Writing file: %s", enum_path)
             f.write(
                 """// Copyright Common Workflow Language project contributors
 """
@@ -593,9 +607,8 @@ public enum {clazz} {{
 """.format(
             fieldname=fieldname, field_doc_str=doc_to_doc_string(doc, indent_level=1)
         )
-        with open(
-            os.path.join(self.main_src_dir, f"{self.current_class}.java"), "a"
-        ) as f:
+        target = self.main_src_dir / f"{self.current_class}.java"
+        with open(target, "a") as f:
             f.write(
                 """
 {getter_doc_str}
@@ -788,7 +801,7 @@ public enum {clazz} {{
         pd = pd + " for parsing documents corresponding to the "
         pd = pd + str(self.base_uri) + " schema."
 
-        template_vars = dict(
+        template_vars: MutableMapping[str, str] = dict(
             base_uri=self.base_uri,
             package=self.package,
             group_id=self.package,
@@ -798,43 +811,32 @@ public enum {clazz} {{
             project_description=pd,
             license_name="Apache License, Version 2.0",
             license_url="https://www.apache.org/licenses/LICENSE-2.0.txt",
-        )  # type: MutableMapping[str, str]
+        )
 
-        def template_from_resource(resource):
-            # type: (str) -> string.Template
+        def template_from_resource(resource: str) -> string.Template:
             template_str = pkg_resources.resource_string(
                 __name__, f"java/{resource}"
             ).decode("utf-8")
             template = string.Template(template_str)
             return template
 
-        def expand_resource_template_to(resource, path):
-            # type: (str, str) -> None
+        def expand_resource_template_to(resource: str, path: Path) -> None:
             template = template_from_resource(resource)
             src = template.safe_substitute(template_vars)
             _ensure_directory_and_write(path, src)
 
-        expand_resource_template_to("pom.xml", os.path.join(self.target_dir, "pom.xml"))
-        expand_resource_template_to(
-            "gitignore", os.path.join(self.target_dir, ".gitignore")
-        )
-        expand_resource_template_to(
-            "package.html", os.path.join(self.main_src_dir, "package.html")
-        )
+        expand_resource_template_to("pom.xml", self.target_dir / "pom.xml")
+        expand_resource_template_to("gitignore", self.target_dir / ".gitignore")
+        expand_resource_template_to("package.html", self.main_src_dir / "package.html")
         expand_resource_template_to(
             "overview.html",
-            os.path.join(self.target_dir, "src", "main", "javadoc", "overview.html"),
+            self.target_dir / "src" / "main" / "javadoc" / "overview.html",
         )
         expand_resource_template_to(
             "MANIFEST.MF",
-            os.path.join(
-                self.target_dir, "src", "main", "resources", "META-INF", "MANIFEST.MF"
-            ),
-        )
-        expand_resource_template_to(
-            "README.md",
-            os.path.join(self.target_dir, "README.md"),
+            self.target_dir / "src" / "main" / "resources" / "META-INF" / "MANIFEST.MF",
         )
+        expand_resource_template_to("README.md", self.target_dir / "README.md")
 
         vocab = ""
         rvocab = ""
@@ -851,13 +853,14 @@ public enum {clazz} {{
         example_tests = ""
         if self.examples:
             _safe_makedirs(self.test_resources_dir)
-            utils_resources = os.path.join(self.test_resources_dir, "utils")
+            utils_resources = self.test_resources_dir / "utils"
             if os.path.exists(utils_resources):
                 shutil.rmtree(utils_resources)
             shutil.copytree(self.examples, utils_resources)
             for example_name in os.listdir(self.examples):
                 if example_name.startswith("valid"):
                     basename = os.path.basename(example_name).rsplit(".", 1)[0]
+                    basename = re.sub(BASIC_JAVA_IDENTIFIER_RE, "_", basename)
                     example_tests += """
   @org.junit.Test
   public void test{basename}ByString() throws Exception {{
@@ -883,11 +886,11 @@ public enum {clazz} {{
     doc = (java.util.Map<String, Object>) YamlUtils.mapFromString(yaml);
     RootLoader.loadDocument(doc, url.toString());
   }}""".format(
-                        basename=basename.replace("-", "_").replace(".", "_"),
+                        basename=basename,
                         example_name=example_name,
                     )
 
-        template_args = dict(
+        template_args: MutableMapping[str, str] = dict(
             package=self.package,
             vocab=vocab,
             rvocab=rvocab,
@@ -895,7 +898,7 @@ public enum {clazz} {{
             root_loader_name=root_loader.name,
             root_loader_instance_type=root_loader.instance_type or "Object",
             example_tests=example_tests,
-        )  # type: MutableMapping[str, str]
+        )
 
         util_src_dirs = {
             "main_utils": self.main_src_dir,
@@ -903,7 +906,7 @@ public enum {clazz} {{
         }
         for (util_src, util_target) in util_src_dirs.items():
             for util in pkg_resources.resource_listdir(__name__, f"java/{util_src}"):
-                src_path = os.path.join(util_target, "utils", util)
+                src_path = util_target / "utils" / util
                 src_template = template_from_resource(os.path.join(util_src, util))
                 src = src_template.safe_substitute(template_args)
                 _ensure_directory_and_write(src_path, src)
