@@ -91,9 +91,10 @@ def to_validation_exception(e: MarkedYAMLError) -> ValidationException:
 
     if e.context:
         parent = ValidationException(e.context)
-        mark = e.context_mark
-        parent.file = re.sub(fname_regex, "", mark.name)
-        parent.start = (mark.line + 1, mark.column + 1)
+        context_mark = e.context_mark
+        if context_mark:
+            parent.file = re.sub(fname_regex, "", context_mark.name)
+            parent.start = (context_mark.line + 1, context_mark.column + 1)
         parent.end = None
         parent.children = [exc]
         return parent
@@ -591,9 +592,11 @@ class Loader:
                                 )
                                 v.lc.filename = document.lc.filename
                             else:
+                                sl = SourceLine(document, idmapField, str)
                                 raise ValidationException(
                                     "mapSubject '{}' value '{}' is not a dict "
-                                    "and does not have a mapPredicate.".format(k, v)
+                                    "and does not have a mapPredicate.".format(k, v),
+                                    sl,
                                 )
                         else:
                             v = val
@@ -689,7 +692,7 @@ class Loader:
                         d,
                         loader,
                         document.lc.data[d] if document.lc.data else document.lc,
-                        getattr(document.lc, "filename", None),
+                        getattr(document.lc, "filename", ""),
                     )
                 elif isinstance(datum, CommentedSeq):
                     datum2 = CommentedSeq()
@@ -731,21 +734,21 @@ class Loader:
         self, document: CommentedMap, loader: "Loader", base_url: str
     ) -> str:
         # Expand identifier field (usually 'id') to resolve scope
-        for identifer in loader.identifiers:
-            if identifer in document:
-                if isinstance(document[identifer], str):
-                    document[identifer] = loader.expand_url(
-                        document[identifer], base_url, scoped_id=True
+        for identifier in loader.identifiers:
+            if identifier in document:
+                if isinstance(document[identifier], str):
+                    document[identifier] = loader.expand_url(
+                        document[identifier], base_url, scoped_id=True
                     )
-                    if document[identifer] not in loader.idx or isinstance(
-                        loader.idx[document[identifer]], str
+                    if document[identifier] not in loader.idx or isinstance(
+                        loader.idx[document[identifier]], str
                     ):
-                        loader.idx[document[identifer]] = document
-                    base_url = document[identifer]
+                        loader.idx[document[identifier]] = document
+                    base_url = document[identifier]
                 else:
                     raise ValidationException(
                         "identifier field '{}' must be a string".format(
-                            document[identifer]
+                            document[identifier]
                         )
                     )
         return base_url
@@ -758,17 +761,17 @@ class Loader:
     ) -> None:
         # Resolve scope for identity fields (fields where the value is the
         # identity of a standalone node, such as enum symbols)
-        for identifer in loader.identity_links:
-            if identifer in document and isinstance(
-                document[identifer], MutableSequence
+        for identifier in loader.identity_links:
+            if identifier in document and isinstance(
+                document[identifier], MutableSequence
             ):
-                for n, v in enumerate(document[identifer]):
+                for n, v in enumerate(document[identifier]):
                     if isinstance(v, str):
-                        document[identifer][n] = loader.expand_url(  # type: ignore
+                        document[identifier][n] = loader.expand_url(  # type: ignore
                             v, base_url, scoped_id=True
                         )
-                        if document[identifer][n] not in loader.idx:
-                            loader.idx[cast(str, document[identifer][n])] = v
+                        if document[identifier][n] not in loader.idx:
+                            loader.idx[cast(str, document[identifier][n])] = v
 
     def _normalize_fields(self, document: CommentedMap, loader: "Loader") -> None:
         # Normalize fields which are prefixed or full URIn to vocabulary terms
@@ -874,13 +877,13 @@ class Loader:
             if newctx is not None:
                 loader = newctx
 
-            for identifer in loader.identity_links:
-                if identifer in document:
-                    if isinstance(document[identifer], str):
-                        document[identifer] = loader.expand_url(
-                            document[identifer], base_url, scoped_id=True
+            for identifier in loader.identity_links:
+                if identifier in document:
+                    if isinstance(document[identifier], str):
+                        document[identifier] = loader.expand_url(
+                            document[identifier], base_url, scoped_id=True
                         )
-                        loader.idx[document[identifer]] = document
+                        loader.idx[document[identifier]] = document
 
             metadata = document
             if "$graph" in document:
