@@ -34,7 +34,7 @@ from schema_salad.utils import (
 )
 
 from . import _logger, jsonld_context, ref_resolver, validate
-from .avro.schema import Names, SchemaParseException, make_avsc_object
+from .avro.schema import Names, SchemaParseException, is_subtype, make_avsc_object
 from .exceptions import (
     ClassValidationException,
     SchemaSaladException,
@@ -81,9 +81,15 @@ SALAD_FILES = (
 
 saladp = "https://w3id.org/cwl/salad#"
 
+cached_metaschema: Optional[Tuple[Names, List[Dict[str, str]], Loader]] = None
+
 
 def get_metaschema() -> Tuple[Names, List[Dict[str, str]], Loader]:
     """Instantiate the metaschema."""
+    global cached_metaschema
+    if cached_metaschema is not None:
+        return cached_metaschema
+
     loader = ref_resolver.Loader(
         {
             "Any": saladp + "Any",
@@ -199,7 +205,8 @@ def get_metaschema() -> Tuple[Names, List[Dict[str, str]], Loader]:
         _logger.error("Metaschema error, avro was:\n%s", json_dumps(sch_obj, indent=4))
         raise
     validate_doc(sch_names, j2, loader, strict=True)
-    return (sch_names, j2, loader)
+    cached_metaschema = (sch_names, j2, loader)
+    return cached_metaschema
 
 
 def add_namespaces(
@@ -242,7 +249,14 @@ def load_schema(
 
     metaschema_names, _metaschema_doc, metaschema_loader = get_metaschema()
     if cache is not None:
-        metaschema_loader.cache.update(cache)
+        # we want to replace some items in the cache, so we need to
+        # make a new Loader with an empty index.
+        for k, v in metaschema_loader.cache.items():
+            if k not in cache:
+                cache[k] = v
+        metaschema_loader = Loader(
+            ctx=metaschema_loader.ctx, cache=cache, session=metaschema_loader.session
+        )
     schema_doc, schema_metadata = metaschema_loader.resolve_ref(schema_ref, "")
 
     if not isinstance(schema_doc, MutableSequence):
@@ -532,7 +546,7 @@ def make_valid_avro(
     # Possibly could be integrated into our fork of avro/schema.py?
     if isinstance(items, MutableMapping):
         avro = copy.copy(items)
-        if avro.get("name") and avro.get("inVocab", True):
+        if avro.get("name"):
             if fielddef:
                 avro["name"] = avro_field_name(avro["name"])
             else:
@@ -562,7 +576,7 @@ def make_valid_avro(
         if "symbols" in avro:
             avro["symbols"] = [avro_field_name(sym) for sym in avro["symbols"]]
         return avro
-    if isinstance(items, MutableSequence):
+    if items and isinstance(items, MutableSequence):
         ret = []
         for i in items:
             ret.append(
@@ -617,7 +631,7 @@ def extend_and_specialize(
                 for spec in aslist(stype["specialize"]):
                     specs[spec["specializeFrom"]] = spec["specializeTo"]
 
-            exfields = []  # type: List[str]
+            exfields = []  # type: List[Any]
             exsym = []  # type: List[str]
             for ex in aslist(stype["extends"]):
                 if ex not in types:
@@ -645,8 +659,46 @@ def extend_and_specialize(
 
             if stype["type"] == "record":
                 stype = copy.copy(stype)
-                exfields.extend(stype.get("fields", []))
-                stype["fields"] = exfields
+                combined_fields = []
+                fields = stype.get("fields", [])
+                # We use short names here so that if a type inherits a field
+                # (e.g. Child#id) from a parent (Parent#id) we avoid adding
+                # the same field twice (previously we had just
+                # ``exfields.extends(stype.fields)``).
+                sns_fields = {shortname(field["name"]): field for field in fields}
+                sns_exfields = {
+                    shortname(exfield["name"]): exfield for exfield in exfields
+                }
+
+                # N.B.: This could be simpler. We could have a single loop
+                #       to create the list of fields. The reason for this more
+                #       convoluted solution is to make sure we keep the order
+                #       of ``exfields`` first, and then the type fields. That's
+                #       because we have unit tests that rely on the order that
+                #       fields are written. Codegen output changes as well.
+                #       We are relying on the insertion order preserving
+                #       property of python dicts (i.e. relyig on Py3.5+).
+
+                # First pass adding the exfields.
+                for sn_exfield, exfield in sns_exfields.items():
+                    field = sns_fields.get(sn_exfield, None)
+                    if field is None:
+                        field = exfield
+                    else:
+                        # make sure field name has not been used yet
+                        if not is_subtype(exfield["type"], field["type"]):
+                            raise SchemaParseException(
+                                f"Field name {field['name']} already in use with "
+                                "incompatible type. "
+                                f"{field['type']} vs {exfield['type']}."
+                            )
+                    combined_fields.append(field)
+                # Second pass, now add the ones that are specific to the subtype.
+                for field in sns_fields.values():
+                    if field not in combined_fields:
+                        combined_fields.append(field)
+
+                stype["fields"] = combined_fields
 
                 fieldnames = set()  # type: Set[str]
                 for field in stype["fields"]:
@@ -661,7 +713,7 @@ def extend_and_specialize(
             elif stype["type"] == "enum":
                 stype = copy.copy(stype)
                 exsym.extend(stype.get("symbols", []))
-                stype["symbol"] = exsym
+                stype["symbols"] = exsym
 
             types[stype["name"]] = stype
 
